/*
 * svxpty.c v1.0 - PTY loop daemon for remotetrx/svxlink
 *
 * Author: Marius Petrescu, YO2LOJ, <marius@yo2loj.ro>
 *
 * Usage:
 *
 *        svxpty [--daemon] <txpty,rxpty> [<txpty,rxpty] [...]
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Version History
 * ---------------
 *    1.0     18.Jan.2016    First version
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <poll.h>
#include <unistd.h>


int main(int argc, char**argv)
{
    struct pollfd *pollfd;
    int *sqfd;
    int pair;
    int npairs= 0;
    char *first;
    char *second;
    char rcv;
    int i;

    int daemon = 0;

    if (argc < 2)
    {
	fprintf(stderr, "Usage: svxpty [--daemon] <txpty,rxpty> [<txpty,rxpty] [...]\n");
	exit(1);
    }
    else
    {
	/* find number of pairs */
	for (i=1; i<argc; i++)
	{
	    if (strcmp(argv[i], "--daemon") == 0)
	    {
		daemon = 1;
		continue;
	    }

	    first = argv[i];
	    if ((second = strstr(first, ",")) != NULL)
	    {
		if ((strlen(first) > 2) && (first != second) && (strlen(second) > 1))
		{
		    npairs++;
		}
		else
		{
		    fprintf(stderr, "svxpty: Invalid pair definition \'%s\'\n", first);
		    exit(1);
		}
		
	    }
	}
    }

    /* allocate memory */
    pollfd = malloc(sizeof(struct pollfd) * (npairs));
    sqfd = malloc(sizeof(int) * (npairs));

    if ((NULL == pollfd) || (NULL == sqfd))
    {
	fprintf(stderr, "svxpty: Memory allocation error\n");
    }

    /* parse and populate */
    pair = 0;
    for (i=1; i<argc; i++)
    {
	if (strcmp(argv[i], "--daemon") == 0)
	{
	    continue;
	}

	first = argv[i];
	if ((second = strstr(first, ",")) != NULL)
	{
	    second[0] = 0;
	    second++;

	    pollfd[pair].fd = open(first, O_RDWR);
	    if (-1 == pollfd[pair].fd)
	    {
		fprintf(stderr, "svxpty: Error opening Tx PTY \'%s\'\n", first);
		exit(1);
	    }
	    fcntl(pollfd[pair].fd, F_SETFL, O_NONBLOCK);
	    pollfd[pair].events = POLLIN;
	    pollfd[pair].revents = 0;

	    sqfd[pair] = open(second, O_RDWR);
	    if (-1 == sqfd[pair])
	    {
		fprintf(stderr, "svxpty: Error opening Rx PTY \'%s\'\n", second);
		exit(1);
	    }
	    pair++;
	}
    }

    /* close all squelches */
    for (pair=0; pair<npairs; pair++)
    {
	write(sqfd[pair], "Z", 1);
    }

    /* daemon if requested */ 
    if (1 == daemon)
    {
	pid_t fork_res = -1;
	fork_res = fork();

	if (-1 == fork_res)
	{
	    fprintf(stderr, "svxpty: Can not become a daemon.\n");
	    exit(1);
	}

	if (0 != fork_res)
	{
	    /* exit parent */
	    exit(0);
	}
    }

    /* run loop */
    while(1)
    {

	while (poll(pollfd, npairs, -1) >= 0)
	{
	    for (pair=0; pair<npairs; pair++)
	    {
		if (pollfd[pair].revents)
		{
		    rcv = 0;
		    if (read(pollfd[pair].fd, &rcv, 1)>0)
		    {
			if (rcv == 'T') {
			    write(sqfd[pair], "O", 1);
			}
			else
			{
			    write(sqfd[pair], "Z", 1);
			}
		    }
		}
	    }
	}
    }
    exit(0);	/* we never reach this */
}
